import click
import pypelinez.git as git
from pypelinez.click_ext import assertUsage


@click.group()
def feature():
    pass


@feature.command()
def name():
    """Adds and commits all changes to branch feature/<name>"""

    print(git.current_branch())


@feature.command()
@click.argument("name")
def start(name):
    """Creates a new branch with the format feature/<name>

    Parameters:
    name (str): feature branch name

    """

    if git.current_branch() == "develop":
        git.create_branch("feature/" + name)
    else:
        print("Must create feature from develop branch")

    assertUsage(git.current_branch() == "develop", "Must create feature from develop branch")

    click.echo("Creating feature: feature/" + name)

    git.create_branch("feature/" + name)


@feature.command()
def add_commit():
    """Adds and commits all changes to branch feature/<name>"""

    git.add()

    git.commit()


@feature.command()
def submit():
    """Submits merge request to develop for feature/<name>"""

    merge_request_file = open("MERGE_REQUEST.md", "r")
    merge_description = merge_request_file.read()
    merge_request_file.close()

    git.merge_request("develop", merge_description)


@feature.command()
def finish():
    """Finishes feature/<name>

    Assumes merge request was successful.
    Pulls from origin develop.
    Deletes the current feature/<name> branch

    """

    branch = git.current_branch()

    git.change_to_branch("develop")

    git.pull("origin", "develop")

    git.delete(branch)

    git.pull("origin", "develop")



import click
import os
import pypelinez.utilities as utilities


# group
@click.group()
def apple():
    pass


# format
@apple.command()
@click.option("--project-path", default="apple", help="Path for Apple project")
@click.option("--project-name", help="Project Name")
@click.option("--configuration", default=".swift-format.json", help="Format configuration file")
@click.argument("directories", nargs=-1)
def format(project_path, project_name, configuration, directories):

    workingDirectory = os.path.join(project_path, project_name)
    directories = " ".join(directories)

    command = """\
    swift format format -i --configuration {config} -r -p {dirs}
    """.format(
        config=configuration, dirs=directories
    )

    result = utilities.action(command, workingDirectory)
    utilities.print_result(result)
    utilities.assert_result(result)


# lint
@apple.command()
@click.option("--project-path", default="apple", help="Path for Apple project")
@click.option("--project-name", help="Project Name")
@click.option("--configuration", default=".swift-format.json", help="Format configuration file")
@click.argument("directories", nargs=-1)
def lint(project_path, project_name, configuration, directories):

    workingDirectory = os.path.join(project_path, project_name)
    directories = " ".join(directories)

    command = """\
    swift format lint --configuration {config} -r -p {dirs}
    """.format(
        config=configuration, dirs=directories
    )

    result = utilities.action(command, workingDirectory)
    utilities.print_result(result)
    utilities.assert_result(result)


# build
@apple.command()
@click.option("--project-path", default="apple", help="Path for Apple project")
@click.option("--project-name", help="Project Name")
@click.option("--platform", help="App platform")
@click.option("--test-platform", help="App test Platform")
@click.option("--device", help="Device name")
@click.option("--os-version", help="Device name")
def build_and_test(project_path, project_name, platform, test_platform, device, os_version):

    workingDirectory = os.path.join(project_path, project_name)

    testPath = os.path.join(
        "Tests",
        "Results",
        platform,
        os_version,
        device.replace(" ", "_"),
        "Results.xcresult",
    )

    utilities.remove_directory(os.path.join(workingDirectory, testPath))

    command = """\
    xcodebuild clean test -quiet -allowProvisioningUpdates \\
    -project "{project_name}.xcodeproj" -scheme "{project_name} ({platform})" \\
    -destination "platform={test_platform},name={device},OS={os_version}" \\
    -resultBundlePath "{testPath}"
    """.format(
        project_name=project_name,
        platform=platform,
        test_platform=test_platform,
        device=device,
        os_version=os_version,
        testPath=testPath,
    )

    print(workingDirectory)
    print(command)

    result = utilities.action(command, workingDirectory)
    utilities.print_result(result)
    utilities.assert_result(result)



import click


def assertUsage(state, message):
    if state is not True:
        raise click.UsageError(message)


def assertBadParameter(state, message):
    if state is not True:
        raise click.BadParameter(message)


def assertLocalVersionIsLatest(state, message):
    if state is not True:
        raise click.UsageError(message)



import click
import pypelinez.git as git
from pypelinez.click_ext import assertLocalVersionIsLatest, assertUsage, assertBadParameter
import pypelinez.utilities as utilities
from semantic_version import Version


@click.group()
def publish():
    pass


# start
@publish.command()
def start():

    assertUsage(git.current_branch() == "develop", "Must publish from a develop branch")

    latest_local_version = git.get_current_local_tag()

    click.echo("Latest local version is " + latest_local_version)

    assertBadParameter(
        utilities.validate_semantic_version(latest_local_version), "Version must follow Semantic Versioning rules"
    )

    remote_versions = git.get_current_remote_tags()

    if len(remote_versions) == 0:

        click.echo("No remote versions")

    else:
        latest_remote_version = remote_versions[0]

        click.echo("Latest remote version is " + latest_remote_version)

        assertLocalVersionIsLatest(
            Version(latest_local_version) > Version(latest_remote_version),
            "Latest remote version ["
            + latest_remote_version
            + "] is higher than latest local version ["
            + latest_local_version
            + "]",
        )

    click.echo("Creating publish request")

    git.create_branch("publish/" + latest_local_version)

    git.merge_request("main", "Publishing " + latest_local_version)
